==> nvdutils-0.2.1.tar.gz: nvdutils-0.2.1/nvdutils/core/loaders/json_loader.py <==
import json

from pathlib import Path

from nvdutils.types.cve import CVE, Description, Reference
from nvdutils.types.options import CVEOptions
from nvdutils.core.loaders.base import CVEDataLoader
from nvdutils.core.parse import parse_weaknesses, parse_metrics, parse_configurations

# TODO: use pydantic for parsing
NVD_JSON_KEYS = ['id', 'descriptions', 'references', 'metrics', 'references']


class JSONFeedsLoader(CVEDataLoader):
    def __init__(self, data_path, options: CVEOptions, **kwargs):
        super().__init__(data_path, options, **kwargs)

    @staticmethod
    def load_cve(path: str) -> CVE:
        path = Path(path)

        if not path.exists():
            raise FileNotFoundError(f"{path} does not exist")
        if path.suffix != '.json':
            raise ValueError(f"{path} is not a json file")
        if path.stat().st_size == 0:
            raise ValueError(f"{path} is empty")

        # read contents of the file
        with path.open('r') as f:
            cve_data = json.load(f)

        # check if the file has the expected keys
        if not all(key in cve_data for key in NVD_JSON_KEYS):
            raise ValueError(f"{path} does not have the expected keys")

        descriptions = [Description(**desc) for desc in cve_data['descriptions']]

        weaknesses = parse_weaknesses(cve_data['weaknesses']) if 'weaknesses' in cve_data else []
        metrics = parse_metrics(cve_data['metrics']) if 'metrics' in cve_data else []
        configurations = parse_configurations(cve_data['configurations']) if 'configurations' in cve_data else []
        references = [Reference(**ref) for ref in cve_data['references']]

        cve = CVE(id=cve_data['id'], status=cve_data.get('vulnStatus', None), weaknesses=weaknesses, metrics=metrics,
                  configurations=configurations, descriptions=descriptions, references=references)

        return cve

    def get_cve_path(self, cve_id: str):
        _, year, number = cve_id.split('-')

        cve_year = f"CVE-{year}"
        length = len(number)

        # Use a range check for better clarity
        if 4 <= length <= 7:
            path = self.data_path / cve_year / f"{cve_year}-{number[:length - 2]}xx" / f"{cve_id}.json"

            if path.exists():
                return path

        raise FileNotFoundError(f"Path for {cve_id} not found")

    def get_cve_ids_by_year(self, year: int) -> list:
        nvd_database_year_path = self.data_path / f"CVE-{year}"

        for path in nvd_database_year_path.iterdir():
            if path.is_dir():
                for cve in path.iterdir():
                    if cve.is_file():
                        yield cve.stem

==> nvdutils-0.4.0.tar.gz: nvdutils-0.4.0/nvdutils/core/loaders/json_loader.py <==
import json

from pathlib import Path

from nvdutils.types.cve import CVE, Description, Reference
from nvdutils.types.options import CVEOptions
from nvdutils.core.loaders.base import CVEDataLoader
from nvdutils.core.parse import parse_weaknesses, parse_metrics, parse_configurations

# TODO: use pydantic for parsing
NVD_JSON_KEYS = ['id', 'descriptions', 'references', 'metrics', 'references']


class JSONFeedsLoader(CVEDataLoader):
    def __init__(self, data_path, options: CVEOptions, **kwargs):
        super().__init__(data_path, options, **kwargs)

    @staticmethod
    def load_cve(path: str) -> CVE:
        path = Path(path)

        if not path.exists():
            raise FileNotFoundError(f"{path} does not exist")
        if path.suffix != '.json':
            raise ValueError(f"{path} is not a json file")
        if path.stat().st_size == 0:
            raise ValueError(f"{path} is empty")

        # read contents of the file
        with path.open('r') as f:
            cve_data = json.load(f)

        # check if the file has the expected keys
        if not all(key in cve_data for key in NVD_JSON_KEYS):
            raise ValueError(f"{path} does not have the expected keys")

        descriptions = [Description(**desc) for desc in cve_data['descriptions']]
        source = cve_data.get('sourceIdentifier', None)
        weaknesses = parse_weaknesses(cve_data['weaknesses']) if 'weaknesses' in cve_data else []
        metrics = parse_metrics(cve_data['metrics']) if 'metrics' in cve_data else []
        configurations = parse_configurations(cve_data['configurations']) if 'configurations' in cve_data else []
        references = [Reference(**ref) for ref in cve_data['references']]

        cve = CVE(id=cve_data['id'], source=source, status=cve_data.get('vulnStatus', None), weaknesses=weaknesses,
                  metrics=metrics, configurations=configurations, descriptions=descriptions, references=references)

        return cve

    def get_cve_path(self, cve_id: str):
        _, year, number = cve_id.split('-')

        cve_year = f"CVE-{year}"
        length = len(number)

        # Use a range check for better clarity
        if 4 <= length <= 7:
            path = self.data_path / cve_year / f"{cve_year}-{number[:length - 2]}xx" / f"{cve_id}.json"

            if path.exists():
                return path

        raise FileNotFoundError(f"Path for {cve_id} not found")

    def get_cve_ids_by_year(self, year: int) -> list:
        nvd_database_year_path = self.data_path / f"CVE-{year}"

        for path in nvd_database_year_path.iterdir():
            if path.is_dir():
                for cve in path.iterdir():
                    if cve.is_file():
                        yield cve.stem
